#include <iostream>
#include <stdexcept>
#include <vector>
#include <fstream>
#include <algorithm>
#include "ido.h"


using namespace std;

struct Use {
    IdoPont kezd;
    IdoPont veg;
    IdoTartam t;
    int hely;
};

struct Vendeg {
    int azon;
    IdoPont kezd;
    IdoPont veg;
    IdoTartam t;
    vector<Use> lista;
};

const int N = 100; //meresi adatok szama;

Vendeg adatok[N];
string helyek[5] = {"Oltozo","Uszoda", "Szaunak", "Gyogyvizes medencek", "Strand"};

void Input(const char* filenev) {
    ifstream is(filenev);
    int azon, hely, qka;

    for (int i = 0; i < 100; ++i) {
        is >> adatok[i].azon;
        is>> qka >> qka >> adatok[i].kezd;

        while(is>>azon && is>>hely && hely!=0) {
            Use u;
            u.hely = hely;
            is>> qka >> u.kezd;
            is>>qka >> qka >> qka >> u.veg;
            u.t = u.veg-u.kezd;
            adatok[i].lista.push_back(u);
        }
        is >> qka >> adatok[i].veg;
        adatok[i].t = adatok[i].veg - adatok[i].kezd;
    }
    is.close();
}

int Egyhelyi() {
    int db = 0;
    for (int i = 0; i < N; ++i)
        if (adatok[i].lista.size() == 1)
            ++db;
    return db;
}

int MaxHossz() {
    int maxVendeg = 0;
    for (int i = 1; i < N; ++i)
        if(adatok[maxVendeg].t < adatok[i].t)
            maxVendeg = i;
    return maxVendeg;
}

int Vendegszam(IdoPont tol, IdoPont ig) {
    int db=0;
    for (int i = 0; i < N; ++i)
        if (adatok[i].kezd >= tol && adatok[i].kezd <= ig)
            db++;
    return db;
}

void Szauna() {
    ofstream os("szauna.txt");
    for(int i = 0; i < N; ++i) {
        IdoTartam T = 0;
       /* imperativ felteteles osszegzes tetel*/
/*
        for (int j = 0; j < adatok[i].lista.size(); ++j)
            if(adatok[i].lista[j].hely == 2)
                T += adatok[i].lista[j].t;
*/
        /* deklarativ Lambda */

        for_each(adatok[i].lista.begin(), adatok[i].lista.end(),
        [&](Use u) {
            if (u.hely == 2) T += u.t;
        });

        if (T > 0) {
            os << adatok[i].azon << " " << T <<endl;
        }
    }
    os.close();
}

void Eloszlas() {
    int db[5] = {0};

    for (int i = 0; i < N; ++i)
        for(int j = 1; j < 5; ++j) {
            size_t ez = 0;
            while(ez < adatok[i].lista.size() && !(adatok[i].lista[ez].hely == j)) ez++;
            if(ez < adatok[i].lista.size() )
                db[j]++;
        }
    for(int j = 1; j < 5; ++j) {
        cout << helyek[j] << ": " << db[j]<< endl;
    }
}

int main() {
    Input("furdoadat.txt");

    cout << "2. feladat" << endl;
    cout << "Az elso vendeg " << adatok[0].kezd << "-kor lepett ki az oltozobol." << endl;
    cout << "Az utolso vendeg " << adatok[N - 1].kezd << "-kor lepett ki az oltozobol." << endl;


    cout << "3. feladat" << endl;
    cout << "A furdoben " << Egyhelyi() << " vendeg jart csak egy reszlegen." << endl;

    cout << "\tLambdaval" << endl;
    cout << "\tA furdoben "<< count_if(begin(adatok), end(adatok),[](Vendeg v) {
        return v.lista.size()==1;
    }) << " vendeg jart csak egy reszlegen." << endl;


    cout << "4. feladat" << endl;
    int id = MaxHossz();
    cout << "A legtobb idot eltolto vendeg:" << endl;
    cout << adatok[id].azon << ". vendeg " << adatok[id].t << endl;

    cout << "\tLambdaval" << endl;
    auto lambda = max_element(begin(adatok), end(adatok),
    [](Vendeg v1, Vendeg v2) {
        return (v1.t < v2.t);
    });
    cout << "\t" << lambda->azon << ". vendeg " << lambda->t << endl;


    cout << "5. feladat" << endl;
    cout << "6-9 ora kozott " << Vendegszam(IdoPont("06:00:00"), IdoPont("08:59:59")) << " vendeg" << endl;
    cout << "9-16 ora kozott " << Vendegszam(IdoPont("09:00:00"), IdoPont("15:59:59")) << " vendeg" << endl;
    cout << "16-20 ora kozott " << Vendegszam(IdoPont("16:00:00"), IdoPont("19:59:59")) << " vendeg" << endl;

    cout << "\tLambdaval" << endl;
    cout << "\t6-9 ora kozott "<< count_if(begin(adatok), end(adatok),[](Vendeg v) {
        return IdoPont("06:00:00")<= v.kezd && v.kezd < IdoPont("09:00:00");
    }) << " vendeg" << endl;
    cout << "\t9-16 ora kozott "<< count_if(begin(adatok), end(adatok),[](Vendeg v) {
        return IdoPont("09:00:00")<= v.kezd && v.kezd < IdoPont("16:00:00");
    }) << " vendeg" << endl;
    cout << "\t16-20 ora kozott "<< count_if(begin(adatok), end(adatok),[](Vendeg v) {
        return IdoPont("16:00:00")<= v.kezd && v.kezd < IdoPont("20:00:00");
    }) << " vendeg" << endl;

    /*6. feladat*/
    Szauna();

    cout << "7. feladat" << endl;
    Eloszlas();
    cout << "\tLambdakkal" << endl;
    for (int hely = 1; hely < 5; ++hely)
        cout << "\t" << helyek[hely] << ": " << count_if(begin(adatok), end(adatok),
        [=](Vendeg v) {
        return find_if(v.lista.begin(), v.lista.end(),
        [=](Use u) {
            return (u.hely == hely);
        })!= v.lista.end();
    }) << endl;

    return 0;
}
